package com.generaldata.skec.cn80listener;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.BatteryManager;
import android.os.Environment;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.EventChannel.EventSink;
import io.flutter.plugin.common.EventChannel.StreamHandler;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import java.util.Map;

/** cn80listenerPlugin */
public class Cn80listenerPlugin implements FlutterPlugin, MethodCallHandler, ScannerCallBack {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private Context applicationContext;
  private BroadcastReceiver scannedCodeReceiver;
  private MethodChannel methodChannel;
  private HoneywellScanner scanner;
  private Handler uiThreadHandler = new Handler(Looper.getMainLooper());

  private static final String _SET_PROPERTIES = "setProperties";
  private static final String _START_SCANNER = "startScanner";
  private static final String _RESUME_SCANNER = "resumeScanner";
  private static final String _PAUSE_SCANNER = "pauseScanner";
  private static final String _STOP_SCANNER = "stopScanner";
  private static final String _ON_DECODED = "onDecoded";
  private static final String _ON_ERROR = "onError";

  @Override
  public void onAttachedToEngine(FlutterPluginBinding binding) {
    Log.e("nutrasys_cn80", "onAttachedToEngine");
    onAttachedToEngine(binding.getApplicationContext(), binding.getBinaryMessenger());
  }

  private void onAttachedToEngine(Context applicationContext, BinaryMessenger messenger) {
    this.applicationContext = applicationContext;
    methodChannel = new MethodChannel(messenger, "cn80listener");
    methodChannel.setMethodCallHandler(this);
    scanner = new HoneywellScannerNative(applicationContext);
    scanner.setScannerCallBack(this);
  }

  public static void registerWith(Registrar registrar) {
    Log.e("nutrasys_cn80", "registerWith");
    final Cn80listenerPlugin instance = new Cn80listenerPlugin();
    instance.onAttachedToEngine(registrar.context(), registrar.messenger());
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {

    switch(call.method)
    {
      case _SET_PROPERTIES:
        if(scanner != null){
          scanner.setProperties((Map<String, Object>) call.arguments);
          result.success(true);
        } else scannerNotInitialized(result);
        break;

      case _START_SCANNER:
        Log.e("nutrasys_cn80", _START_SCANNER);
        if(scanner != null){
          scanner.startScanner();
          Log.e("nutrasys_cn80", "Scanner Started");
          result.success(true);
        } else scannerNotInitialized(result);
        break;
      case _RESUME_SCANNER:
        if(scanner != null){
          scanner.resumeScanner();
          Log.e("nutrasys_cn80", "Scanner Resumed");
          result.success(true);
        } else scannerNotInitialized(result);
        break;
      case _PAUSE_SCANNER:
        if(scanner != null){
          scanner.pauseScanner();
          Log.e("nutrasys_cn80", "Scanner Paused");
          result.success(true);
        } else scannerNotInitialized(result);
        break;
      case _STOP_SCANNER:
        if(scanner != null){
          scanner.stopScanner();
          Log.e("nutrasys_cn80", "Scanner Stopped");
          result.success(true);
        } else scannerNotInitialized(result);
        break;
      default:
        result.notImplemented();
    }
  }

  private void scannerNotInitialized(Result result){
    result.error("Scanner has not been initialized.", null, null);
  }

  @Override
  public void onDetachedFromEngine(FlutterPluginBinding binding) {
    applicationContext = null;
    methodChannel.setMethodCallHandler(null);
    methodChannel = null;
  }

  /**
   * Called when decoder has successfully decoded the code
   * <br>
   * Note that this method always called on a worker thread
   *
   * @param code Encapsulates the result of decoding a barcode within an image
   */
  @Override
  public void onDecoded(final String code)
  {
    Log.e("nutrasys_cn80", "SCAN Recorded: " + code);
    uiThreadHandler.post(new Runnable() {
      @Override
      public void run () {
        methodChannel.invokeMethod("scanDecoded", code);
      }
    });
  }

  /**
   * Called when error has occurred
   * <br>
   * Note that this method always called on a worker thread
   *
   * @param error Exception that has been thrown
   */
  @Override
  public void onError(final Exception error)
  {
    Log.e("nutrasys_cn80", "SCAN Processing ERROR: " + error.getMessage());
    uiThreadHandler.post(new Runnable() {
      @Override
      public void run () {
        methodChannel.invokeMethod("scanDecodedWithError", error.getMessage());
      }
    });
  }
}
